@@ -1,5 +1,5 @@
 /****************************************************************************
- ** $Id: torclient.cpp,v 1.115 2007/12/29 21:18:50 hoganrobert Exp $
+ ** $Id: torclient.cpp,v 1.124 2008/06/08 21:05:11 hoganrobert Exp $
  *   Copyright (C) 2006 by Robert Hogan   *
  *   robert@roberthogan.net   *
  *                                                                         *
@@ -36,6 +36,7 @@
 #include <qfile.h>
 #include <sys/socket.h>
 #include <netdb.h>
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <kstandarddirs.h>
@@ -100,11 +101,11 @@ TorClient::TorClient( const QString &host, Q_UINT16 port )
     serverStatus["BAD_SERVER_DESCRIPTOR"] = "<font color='#990000'>Server Not Accepted By Tor Network Yet.</font>";
     serverStatus["GOOD_SERVER_DESCRIPTOR"] = "Accepted By Tor Network";
 
-    serverStatusIcon["REACHABILITY_SUCCEEDED"] = "tork_greenserver";
-    serverStatusIcon["REACHABILITY_FAILED"] = "tork_orangeserver";
-    serverStatusIcon["BAD_SERVER_DESCRIPTOR"] = "tork_littleserver";
-    serverStatusIcon["GOOD_SERVER_DESCRIPTOR"] = "tork_greenserver";
-    serverStatusIcon["DIR_ALL_UNREACHABLE"] = "tork_redserver";
+    serverStatusIcon["REACHABILITY_SUCCEEDED"] = "tork_green";
+    serverStatusIcon["REACHABILITY_FAILED"] = "tork_orange";
+    serverStatusIcon["BAD_SERVER_DESCRIPTOR"] = "tork_little";
+    serverStatusIcon["GOOD_SERVER_DESCRIPTOR"] = "tork_green";
+    serverStatusIcon["DIR_ALL_UNREACHABLE"] = "tork_red";
 
     clientStatus["NOT_ENOUGH_DIR_INFO"] = "<font color='#990000'>Not Enough Info To Try Network Yet</font>";
     clientStatus["ENOUGH_DIR_INFO"] = "Enough Info To Try Network";
@@ -116,7 +117,18 @@ TorClient::TorClient( const QString &host, Q_UINT16 port )
     clientStatusIcon[clientStatus["CIRCUIT_ESTABLISHED"]] = "tork_green";
     clientStatusIcon[clientStatus["CIRCUIT_NOT_ESTABLISHED"]] = "tork_orange";
 
+    portMessage["23"] = " Port 23 is used by telnet, which transmits usernames "
+                        "and passwords unencrypted.";
+    portMessage["110"] = " Port 110 is used to download email, so your login "
+                        "details can be transmitted unencrypted.";
+    portMessage["109"] = " Port 109 is used to download email, so your login "
+                        "details can be transmitted unencrypted.";
+    portMessage["143"] = " Port 143 is used to download email, so your login "
+                        "details can be transmitted unencrypted.";
 
+
+    statusMessage["DANGEROUS_PORT"] = "QUESTIONMESSAGETraffic on Port PORT "
+                                      "has been rejected by Tor.";
     statusMessage["DANGEROUS_VERSION"] = "QUESTIONMESSAGEYou are using Tor CURRENT."
                                          " This version is REASON. "
                                          "You should use Tor RECOMMENDED instead";
@@ -191,9 +203,25 @@ void TorClient::configureServer( )
         ( sendToServer(QString("SETCONF ExitPolicy=\"%2\"").arg( TorkConfig::exitPolicy().join(","))));
 
     sendToServer(QString("SETCONF NickName=%1").arg(TorkConfig::nickName())) ;
-    sendToServer(QString("SETCONF ORPort=%1").arg(TorkConfig::oRPort())) ;
-    sendToServer(QString("SETCONF DirPort=%1").arg(TorkConfig::dirPort())) ;
-    sendToServer(QString("SETCONF BridgeRelay=%1").arg(TorkConfig::bridgeRelay())) ;
+    if (!TorkConfig::clientOnly()){
+      //We send the orport configs together to avoid error messages from Tor
+      //telling us that one cannot be set without the other.
+      sendToServer(QString("SETCONF ORPort=%1 "
+                           "ORListenAddress=0.0.0.0:%2")
+                           .arg(TorkConfig::oRPort())
+                           .arg(TorkConfig::oRListenAddress())) ;
+      //We send the dirport configs together to avoid error messages from Tor
+      //telling us that one cannot be set without the other.
+      sendToServer(QString("SETCONF DIRPort=%1 "
+                           "DIRListenAddress=0.0.0.0:%2")
+                           .arg(TorkConfig::dirPort())
+                           .arg(TorkConfig::dirListenAddress())) ;
+      sendToServer(QString("SETCONF BridgeRelay=%1").arg(TorkConfig::bridgeRelay())) ;
+    }else{
+      sendToServer(QString("SETCONF ORPort= ORListenAddress=")) ;
+      sendToServer(QString("SETCONF DirPort= DirListenAddress=")) ;
+      sendToServer(QString("SETCONF BridgeRelay=")) ;
+    }
 
     if (TorkConfig::clientOnly())
         resetClientReport();
@@ -280,6 +308,7 @@ void TorClient::fetchServerInfo( const QString & server)
     QString fp = getFPFromFPDigest(server);
     fp.replace("$","");
 
+    kdDebug() << fp << endl;
 //    emit showServerBW(fp);
 
     sendToServer("GETINFO dir/server/fp/" + fp);
@@ -405,10 +434,19 @@ void TorClient::bandwidth()
     sendToServer("GETINFO status/good-server-descriptor");
     sendToServer("GETINFO status/reachability-succeeded/or");
 
+    //Always enable for each session, user can disable through yes/no
+    //interface when warned - but only for that session.
+    m_portsToReject.clear();
+    m_portsToReject << "23" << "109" << "110" << "143";
+    sendToServer(QString("SETCONF WarnPlainTextPorts=%1")
+                 .arg(m_portsToReject.join(",")));
+    sendToServer(QString("SETCONF RejectPlainTextPorts=%1")
+                .arg(m_portsToReject.join(",")));
     sendToServer("SETEVENTS EXTENDED CIRC  STREAM  ORCONN  NOTICE "
                   "WARN  ERR  ADDRMAP BW STREAM_BW NS STATUS_GENERAL "
                   "STATUS_CLIENT STATUS_SERVER GUARD");
 
+
 }
 
 void TorClient::handle250(const QString &lin)
@@ -444,8 +482,6 @@ void TorClient::handle250(const QString &lin)
         m_expectingServerInfo= true;
     else if (line.contains("250+extra-info/digest/"))
         m_expectingServerInfo= true;
-    else if (line.contains("650+NS"))
-        m_expectingDirStatus= true;
     else if (line.contains("250+ns/all=")){
         m_expectingDirStatus= true;
         emit whatImDoing("Inspecting the Tor network..");
@@ -505,7 +541,7 @@ void TorClient::socketReadyRead()
 
         if (m_expectingDirStatus){
             if (!(line == ".")){
-                parseEvent("DIRSTATUS",line);
+                parseDirStatus(line);
             }else{
                 m_expectingDirStatus = false;
                 sendToServer("GETINFO entry-guards");
@@ -559,11 +595,14 @@ void TorClient::socketReadyRead()
         if (code == "250")
             handle250(line);
         else if (code == "650"){
+            if (line.contains("650+NS")){
+                m_expectingDirStatus= true;
+                continue;
+            }
             QString eventType = line.section(" ",1,1);
             QString eventInfo = line.section(" ",2);
             if (eventInfo.contains("circuit_testing_failed"))
                 emit serverError();
-
             parseEvent(eventType,eventInfo);
         }else if (code == "552"){
             QString eventInfo = line.section(" ",1);
@@ -659,6 +698,10 @@ void TorClient::populateMessageFromStatusDetail(const QString &line, QString &me
         QString value = rx.cap(0).section("=",1,1).stripWhiteSpace();
         message.replace(keyword,value);
         pos += rx.matchedLength();
+        if (keyword=="PORT"){
+            m_WarnedPorts << value;
+            message.append(portMessage[value]);
+        }
     }
 }
 
@@ -753,7 +796,7 @@ void TorClient::parseStream(const QString &info)
 void TorClient::parseServer(const QString &info)
 {
 
-//     kdDebug() << info << endl;
+     kdDebug() << "server info " << info << endl;
     if (info.left(7) == "router "){
         QString ip = info.section(" ",2,2);
         QString cc;
@@ -915,7 +958,6 @@ void TorClient::parseCircuit(const QString &info)
     }
     if (!exit.isEmpty())
         exit = getFPDigestFromFP(exit);
-
     //Strip out the FPs from the circuit, if any
     path.replace(QRegExp("(\\$[A-Z0-9]{40})(~|=)"),"");
 
@@ -1355,7 +1397,8 @@ void TorClient::parseDirStatus(const QString &info)
 
     if (info.left(2) == "s "){
 
-        if (QDate::currentDate().daysTo(QDate(ds_date.left(4).toInt(),ds_date.mid(5,2).toInt(),ds_date.right(2).toInt())) > -30) {
+        if (QDate::currentDate().daysTo(QDate(ds_date.left(4).toInt(),
+            ds_date.mid(5,2).toInt(),ds_date.right(2).toInt())) > -30) {
             if (server(ds_identity).isEmpty())
                 storeServer(ds_server,ds_identity);
     
@@ -1376,7 +1419,8 @@ void TorClient::setBandwidth(const QString &rate, const QString &burst, const QS
 {
    ( sendToServer(QString("SETCONF BandwidthRate=%2KB BandwidthBurst=%2KB")
                   .arg(rate).arg(burst)));
-   ( sendToServer(QString("SETCONF MaxAdvertisedBandwidth=%2KB").arg(max)));
+   if (!max.isEmpty())
+    ( sendToServer(QString("SETCONF MaxAdvertisedBandwidth=%2KB").arg(max)));
 }
 
 void TorClient::getBandwidth()
@@ -1403,4 +1447,30 @@ void TorClient::closeAllCircuits( QListView* &circuitList)
         }
 }
 
+void TorClient::assignPortToRemove()
+{
+    //called by the warning message to get the port to remove in case
+    // user requests it.
+    if (m_WarnedPorts[0].isNull())
+      return;
+    kdDebug() << "warnedports " << m_WarnedPorts[0] << endl;
+    m_portToRemove = m_WarnedPorts[0];
+    kdDebug() << "porttoremove" << m_portToRemove << endl;
+    m_WarnedPorts.pop_front();
+    kdDebug() << "porttoremove" << m_portToRemove << endl;
+
+}
+
+void TorClient::allowPlainTextPorts()
+{
+
+    kdDebug() << "m_portsToReject " << m_portsToReject << endl;
+
+    m_portsToReject.remove(m_portsToReject.find(m_portToRemove));
+    sendToServer(QString("SETCONF WarnPlainTextPorts=%1")
+                 .arg(m_portsToReject.join(",")));
+    sendToServer(QString("SETCONF RejectPlainTextPorts=%1")
+                .arg(m_portsToReject.join(",")));
+}
+
 #include "torclient.moc"
