@@ -1,5 +1,5 @@
 /***************************************************************************
- * $Id: update.cpp,v 1.21 2007/12/21 19:14:26 hoganrobert Exp $
+ * $Id: update.cpp,v 1.22 2008/06/02 10:38:27 hoganrobert Exp $
  *   Copyright (C) 2006 by Robert Hogan                                    *
  *   robert@roberthogan.net                                                *
  *                                                                         *
@@ -74,8 +74,8 @@ const char *mirrors[] = {
     0
 };
 
-TorkUpdate::TorkUpdate(QWidget *parent,bool alpha, const char *name)
-    : QWidget(parent, name),m_alpha(alpha),filelist(new KHTMLPart(this))
+TorkUpdate::TorkUpdate(QWidget *parent, const char *name)
+    : QWidget(parent, name),filelist(new KHTMLPart(this))
 {
 
 }
@@ -475,6 +475,7 @@ void TorkUpdate::getLatestVersionFromSF(KURL url)
     connect( filelist, SIGNAL( completed() ), this,
 		SLOT( parseSFPage() ) );
 
+    filelist->setStatusMessagesEnabled(false);
     filelist->openURL(url);
 
 }
@@ -489,6 +490,7 @@ void TorkUpdate::getLatestVersionFromSFHack(KURL url)
     connect( filelist, SIGNAL( completed() ), this,
 		SLOT( parseSFPageHack() ) );
 
+    filelist->setStatusMessagesEnabled(false);
     filelist->openURL(url);
 
 }
@@ -587,8 +589,7 @@ void TorkUpdate::getVersionFromLink( const DOM::Node &n )
         
         QString rawversion = version;
 
-        if (m_alpha)
-            version.remove("-alpha").remove("-beta").remove("-rc");
+        version.remove("-alpha").remove("-beta").remove("-rc");
 
         numericversion = numericizeVersion(version);
 
